unit SafeUpAndDown.UpAndDownThread;

interface

uses
  System.Classes, System.SysUtils;

type
  TUpAndDownThread = class(TThread)
  private
    FLoops: Integer;
    FSyncedAccess: Boolean;
    FCountUp: Boolean;
    FMainThreadLog: TStrings;
    procedure OutputMessage(const AMessage: string);
    function GetName: string;
  public
    constructor Create(const ALoops: Integer; ACountUp: Boolean; ASyncedAccess: Boolean; ALog: TStrings);
    procedure Execute; override;
  end;

var
  GlobalCounter: NativeInt = 0;

implementation

uses
  Winapi.Windows, System.Diagnostics;


{ TUpAndDownThread }

constructor TUpAndDownThread.Create(const ALoops: Integer; ACountUp: Boolean; ASyncedAccess: Boolean; ALog: TStrings);
begin
  FMainThreadLog := ALog;
  FLoops := ALoops;
  FCountUp := ACountUp;
  FSyncedAccess := ASyncedAccess;
  FreeOnTerminate := true;
  inherited Create(false);
end;


procedure TUpAndDownThread.OutputMessage(const AMessage: string);
begin
  if Assigned(FMainThreadLog) then
    FMainThreadLog.Add(AMessage);
end;


function TUpAndDownThread.GetName: string;
const
  sNames : Array[Boolean] of String =
     ('Runterzhlender Thread','Hochzhlender Thread');
  sSynced : Array[Boolean] of String =
     ('nicht synchronsiert','synchronisiert');

begin
  Result := sNames[FCountUp] + ' ' + sSynced[FSyncedAccess];
end;


procedure TUpAndDownThread.Execute;
var
  index: Integer;
  stopWatch: TStopwatch;
begin
  Synchronize(procedure
              begin
                OutputMessage(GetName + ' startet');
              end);

  stopWatch := TStopwatch.StartNew;
  for index := 1 to FLoops do
  begin
    if FCountUp then
    begin
      if FSyncedAccess then
        AtomicIncrement(GlobalCounter)
      else
        Inc(GlobalCounter);
    end
    else
    begin
      if FSyncedAccess then
        AtomicDecrement(GlobalCounter)
      else
        Dec(GlobalCounter);
    end;
  end;
  stopWatch.Stop;

  Synchronize(procedure
              begin
                OutputMessage(Format('%s fertig nach %d Millisekunden',
                              [GetName, stopWatch.ElapsedMilliseconds]));
              end);
end;



end.
