unit NoSync.UpAndDownThread;

interface

uses
  System.Classes, System.SysUtils;

type
  TUpAndDownThread = class(TThread)
  private
    FLoops: Integer;
    FCountUp: Boolean;
    FMainThreadLog: TStrings;
    procedure OutputMessage(const AMessage: string);
    function GetName: string;
  public
    constructor Create(const ALoops: Integer; ACountUp: Boolean; ALog: TStrings);
    procedure Execute; override;
  end;

var
  GlobalCounter: NativeInt = 0;

implementation

uses
   System.Diagnostics;


{ TPrjClassUpAndDown }

constructor TUpAndDownThread.Create(const ALoops: Integer; ACountUp: Boolean; ALog: TStrings);
begin
  FMainThreadLog := ALog;
  FLoops := ALoops;
  FCountUp := ACountUp;
  FreeOnTerminate := true;
  inherited Create(false);
end;


procedure TUpAndDownThread.OutputMessage(const AMessage: string);
begin
  if Assigned(FMainThreadLog) then
    FMainThreadLog.Add(AMessage);
end;


function TUpAndDownThread.GetName: string;
const
  sNames: Array[Boolean] of String =
     ('Runterzhlender Thread','Hochzhlender Thread');
begin
  Result := sNames[FCountUp];
end;


procedure TUpAndDownThread.Execute;
var
  index: Integer;
  stopWatch: TStopwatch;
begin
  Synchronize(procedure
              begin
                OutputMessage(GetName + ' startet');
              end);

  stopWatch := TStopwatch.StartNew;
  for index := 1 to FLoops do
  begin
    if FCountUp then
      Inc(GlobalCounter)
    else
      Dec(GlobalCounter);
  end;
  stopWatch.Stop;

  Synchronize(procedure
              begin
                OutputMessage(Format('%s fertig nach %d Millisekunden',
                              [GetName, stopWatch.ElapsedMilliseconds]));
              end);
end;



end.
