unit SyncMutex.UpAndDownThread;

interface

uses
  System.SyncObjs, System.Classes, System.SysUtils;

type
  TUpAndDownThread = class(TThread)
  private
    FLoops: Integer;
    FMutex: TMutex;
    FCountUp: Boolean;
    FMainThreadLog: TStrings;
    procedure OutputMessage(const AMessage: string);
    function GetName: string;
  public
    constructor Create(const ALoops: Integer; ACountUp: Boolean; ALog: TStrings);
    procedure Execute; override;
  end;

var
  GlobalCounter: NativeInt = 0;

implementation

uses
  System.StrUtils, System.Diagnostics;


{ TPrjClassUpAndDown }

constructor TUpAndDownThread.Create(const ALoops: Integer; ACountUp: Boolean; ALog: TStrings);
begin
  FMainThreadLog := ALog;
  FLoops := ALoops;
  FCountUp := ACountUp;
  FMutex := TMutex.Create(nil, false, 'UpAndDownMutex'+ MainInstance.ToString );
  FreeOnTerminate := true;
  inherited Create(false);
end;


procedure TUpAndDownThread.OutputMessage(const AMessage: string);
begin
  if Assigned(FMainThreadLog) then
    FMainThreadLog.Add(AMessage);
end;


function TUpAndDownThread.GetName: string;
const
  sNames : Array[Boolean] of String =
     ('Runterzhlender Thread','Hochzhlender Thread');
begin
  Result := sNames[FCountUp] + ' Mutex, benannt';
end;


procedure TUpAndDownThread.Execute;
var
  index: Integer;
  stopWatch: TStopwatch;
begin
  Synchronize(procedure
              begin
                OutputMessage(GetName + ' startet');
              end);

  stopWatch := TStopwatch.StartNew;
  for index := 1 to FLoops do
  begin
    if FCountUp then
    begin
      FMutex.Acquire;
      try
        Inc(GlobalCounter);
      finally
        FMutex.Release;
      end;
    end
    else
    begin
      FMutex.Acquire;
      try
        Dec(GlobalCounter);
      finally
        FMutex.Release;
      end;
    end;
  end;
  stopWatch.Stop;

  Synchronize(procedure
              begin
                OutputMessage(Format('%s fertig nach %d Millisekunden',
                              [GetName, stopWatch.ElapsedMilliseconds]));
              end);
end;



end.
