unit SyncMutexAPI.UpAndDownThread;

interface

uses
  System.SyncObjs, System.Classes, System.SysUtils;

type
  TPrjClassUpAndDown = class(TThread)
  private
    FLoops: Integer;
    FMutexHandle: THandle;
    FCountUp: Boolean;
    FMainThreadLog: TStrings;
    procedure OutputMessage(const AMessage: string);
    function GetName: string;
  public
    constructor Create(const ALoops: Integer; ACountUp: Boolean; AMutexHandle: THandle; ALog: TStrings);
    procedure Execute; override;
  end;

var
  GlobalCounter: NativeInt = 0;

implementation

uses
  WinApi.Windows, System.StrUtils, System.Diagnostics;


{ TPrjClassUpAndDown }

constructor TPrjClassUpAndDown.Create(const ALoops: Integer; ACountUp: Boolean; AMutexHandle: THandle; ALog: TStrings);
begin
  FMainThreadLog := ALog;
  FLoops := ALoops;
  FCountUp := ACountUp;
  FMutexHandle := AMutexHandle;
  FreeOnTerminate := true;
  inherited Create(false);
end;


procedure TPrjClassUpAndDown.OutputMessage(const AMessage: string);
begin
  if Assigned(FMainThreadLog) then
    FMainThreadLog.Add(AMessage);
end;


function TPrjClassUpAndDown.GetName: string;
const
  sNames : Array[Boolean] of String =
     ('Runterzhlender Thread','Hochzhlender Thread');
begin
  Result := sNames[FCountUp] + ' Mutex (API, unbenannt)';
end;


procedure TPrjClassUpAndDown.Execute;
var
  index: Integer;
  stopWatch: TStopwatch;
begin
  Synchronize(procedure
              begin
                OutputMessage(GetName + ' startet');
              end);

  stopWatch := TStopwatch.StartNew;
  for index := 1 to FLoops do
  begin
    if FCountUp then
    begin
      if WaitForSingleObject(FMutexHandle, INFINITE) = WAIT_OBJECT_0 then
      try
        Inc(GlobalCounter);
      finally
        ReleaseMutex(FMutexHandle);
      end;
    end
    else
    begin
      if WaitForSingleObject(FMutexHandle, INFINITE) = WAIT_OBJECT_0 then
      try
        Dec(GlobalCounter);
      finally
        ReleaseMutex(FMutexHandle);
      end;
    end;
  end;
  stopWatch.Stop;

  Synchronize(procedure
              begin
                OutputMessage(Format('%s fertig nach %d Millisekunden',
                              [GetName, stopWatch.ElapsedMilliseconds]));
              end);
end;



end.
