unit ZipperSample.FormZip;

interface

uses
  Winapi.Windows, Winapi.Messages,
  System.SysUtils, System.Variants, System.Classes, System.Zip,
  System.IOUtils, System.Threading, System.Diagnostics,
  system.Generics.Collections,
  Vcl.Graphics, Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.StdCtrls, Vcl.FileCtrl, Vcl.CheckLst, Vcl.ExtCtrls,
  ZipperSample.SimpleZip, Vcl.ComCtrls;

type
  TFrmZip = class(TForm)
    Panel1: TPanel;
    Zielverzeichnis: TLabel;
    edtDestDir: TEdit;
    Panel2: TPanel;
    btnCompressNoThread: TButton;
    btnCancel: TButton;
    btnCompressTasks: TButton;
    btnCompressThreads: TButton;
    stbBottom: TStatusBar;
    Panel3: TPanel;
    Label1: TLabel;
    edtSourceDir: TEdit;
    btnSelectDir: TButton;
    procedure FormCreate(Sender: TObject);
    procedure btnCompressNoThreadClick(Sender: TObject);
    procedure btnCompressTasksClick(Sender: TObject);
    procedure btnCompressThreadsClick(Sender: TObject);
    procedure edtDestDirChange(Sender: TObject);
    procedure edtSourceDirChange(Sender: TObject);
    procedure btnSelectDirClick(Sender: TObject);
  private
    FWorking: Boolean;
    FWorktime: TStopwatch;
    FDestDirExists: Boolean;
    FSourceDirExists: Boolean;
    procedure SetWorking(const Value: Boolean);
    procedure CheckActions;
    procedure HandleTerminate(sender: Tobject);
  public
    property Working: Boolean read FWorking write SetWorking;
  end;

var
  FrmZip: TFrmZip;

implementation

{$R *.dfm}

uses
  System.Math;

const
  ZipExt = '.zip';

procedure TFrmZip.CheckActions;
var
  LCompressionEnabled: Boolean;
begin
  LCompressionEnabled := FDestDirExists and FSourceDirExists and (not FWorking);
  btnCompressNoThread.Enabled := LCompressionEnabled;
  btnCompressTasks.Enabled := LCompressionEnabled;
  btnCompressThreads.Enabled := LCompressionEnabled;
end;


procedure TFrmZip.FormCreate(Sender: TObject);
begin
  edtDestDir.Text := TPath.GetTempPath;
  edtSourceDir.Text := TPath.GetDocumentsPath;
end;


procedure TFrmZip.SetWorking(const Value: Boolean);
const
  cStatusTexts : Array[Boolean] of string = ('idle', 'working');
begin
  if FWorking <> Value then
  begin
    FWorking := Value;
    btnCompressNoThread.Enabled := not FWorking;
    btnCancel.Enabled := FWorking;
    edtDestDir.Enabled := not Working;
    edtSourceDir.Enabled := not Working;
    if FWorking then
    begin
      FWorktime.Reset;
      FWorktime.Start;
      stbBottom.Panels[1].Text := '';
    end
    else
    begin
      FWorktime.Stop;
      stbBottom.Panels[1].Text := Format('Bentigte Zeit: %d Sekunden', [FWorktime.Elapsed.Seconds]);
    end;
    stbBottom.Panels[0].Text := cStatusTexts[FWorking];
    Application.ProcessMessages;
    CheckActions;
  end;
end;


procedure TFrmZip.btnCompressNoThreadClick(Sender: TObject);
var
  LZipName: string;
begin
  Working := true;
  try
    LZipName := ExtractFilename(edtSourceDir.Text+ZipExt);
    TZipFile.ZipDirectoryContents(TPath.Combine(edtDestDir.Text,LZipName),edtSourceDir.Text);
  finally
    Working := false;
  end;
end;


procedure TFrmZip.btnCompressTasksClick(Sender: TObject);
var
  LZipName, LSourceDir, LDestDir: string;
begin
  Working := true;
  LSourceDir := edtSourceDir.Text;
  LDestDir := edtDestDir.Text;
  LZipName := ExtractFilename(edtSourceDir.Text+ZipExt);
  TTask.Run(procedure
            begin
              TZipFile.ZipDirectoryContents(TPath.Combine(LDestDir,LZipName),LSourceDir);
              TThread.Queue(nil,procedure
                                      begin
                                         Working := false;
                                      end)
            end)

end;

procedure TFrmZip.HandleTerminate(Sender: TObject);
begin
  Working := false;
end;


procedure TFrmZip.btnCompressThreadsClick(Sender: TObject);
var
  LZipName: string;
begin
  Working := true;
  LZipName := ExtractFilename(edtSourceDir.Text+ZipExt);
  TSimpleZipThread.Create(edtSourceDir.Text,TPath.Combine(edtDestDir.Text,LZipName),HandleTerminate);
end;

procedure TFrmZip.btnSelectDirClick(Sender: TObject);
var
  directory: string;
begin
  directory := edtSourceDir.Text;
  if SelectDirectory('Quellverzeichnis whlen', '', directory) then
  begin
    edtSourceDir.Text := directory;
  end;

end;

procedure TFrmZip.edtDestDirChange(Sender: TObject);
begin
  FDestDirExists := TDirectory.Exists(edtDestDir.Text);
  CheckActions;
end;

procedure TFrmZip.edtSourceDirChange(Sender: TObject);
begin
  FSourceDirExists := TDirectory.Exists(edtSourceDir.Text);
  CheckActions;
end;

end.
